package be.jaud;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.InvocationTargetException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.sql.*;

public class MariaDB {

    private static Connection conn = null;
    private static MariaDB self = null;
    private static Settings conf = Settings.getInstance();
    public static final Logger LOG = LoggerFactory.getLogger(MariaDB.class);
    private MariaDB(String Host,String db,String User,String Password) {
        try {

            // Datenbanktreiber fuer ODBC Schnittstellen laden.
            // F<>r verschiedene ODBC-Datenbanken muss dieser Treiber
            // nur einmal geladen werden.
            Class.forName("org.mariadb.jdbc.Driver").getDeclaredConstructor().newInstance();

            // Verbindung zur ODBC-Datenbank herstellen.
            // Es wird die JDBC-ODBC-Bruecke verwendet.
            // Port -- Standard: 3306
            String dbPort = "3306";
            conn = DriverManager.getConnection("jdbc:mariadb://" + Host + ":"
                    + dbPort + "/" + db + "?" + "user=" + URLEncoder.encode(User, StandardCharsets.UTF_8) + "&"
                    + "password=" + Password + "&useSSL=false&autoReconnect=true&useJDBCCompliantTimezoneShift=true&useLegacyDatetimeCode=false&serverTimezone=UTC");
        } catch (ClassNotFoundException | SQLException | IllegalAccessException | InstantiationException |
                 NoSuchMethodException | InvocationTargetException e) {
            LOG.error("SQLException: {}", e.getMessage(), e);
            System.exit(1);
        }
    }

    public static Connection getInstance()
    {
        try {
            if(conn == null || !conn.isValid(0)){
                self = new MariaDB(conf.getString("db.server") , conf.getString("db.database"), conf.getString("db.user"), conf.getString("db.password"));
            }
        } catch (SQLException e) {
            LOG.error("SQLException: {}", e.getMessage(), e);
        }
        return conn;
    }

    /**
     * Returns the Connection
     * @return MySQLConnection
     * @deprecated
     */
    public static MariaDB getDB()
    {
        try {
            if(conn == null || !conn.isValid(0)){
                self = new MariaDB(conf.getString("db.server") , conf.getString("db.database"), conf.getString("db.user"), conf.getString("db.password"));
            }
        } catch (SQLException e) {
            LOG.error("SQLException: {}", e.getMessage(), e);
        }
        return self;
    }

    public static void close()
    {
        if(conn != null){
            try {
                conn.close();
            } catch (SQLException e) {
                LOG.error("SQLException: {}", e.getMessage(), e);
            }
        }

    }
    public static ResultSet load_users()
    {
        conn = getInstance();

        if(conn != null)
        {
            // Anfrage-Statement erzeugen.
            Statement query;
            try {
                query = conn.createStatement();

                // Ergebnistabelle erzeugen und abholen.
                String sql = "SELECT DISTINCT `user`  FROM `"+ conf.getString("db.table")+"`";
                return query.executeQuery(sql);
            } catch (SQLException e) {
                LOG.error("SQLException: {}", e.getMessage());
            }
        }
        return null;
    }

    /**
     * Loads the data of a user
     * @param id Userid
     * @return ResultSet
     */
    public static ResultSet load_user(String id)
    {
        conn = getInstance();

        if(conn != null)
        {
            // Anfrage-Statement erzeugen.
            Statement query;
            try {
                query = conn.createStatement();

                // Ergebnistabelle erzeugen und abholen.
                String sql = "SELECT * FROM `"+ conf.getString("db.table")+"` WHERE `user`='"+id+"' ORDER BY `display`";
                return query.executeQuery(sql);
            } catch (SQLException e) {
                LOG.error("SQLException: {}", e.getMessage());
            }
        }
        return null;
    }

}
